# -*- coding: utf-8 -*-
#
# Copyright 2016 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
import os

RQDATAC_DEFAULT_ADDRESS = "rqdatad-pro.ricequant.com:16011"
TAG_MAP = ["stock", "futures", "fund", "index", "option", "convertible", ]
EXTRA_INDEX_URL = "--extra-index-url https://rquser:Ricequant8@pypi2.ricequant.com/simple/"
BASH_FILE = [".bash_profile", ".bashrc", ".bash_profile", ".zshrc"]
VERSION_MAP = {}

version_map_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "version_map.json"))
if os.path.exists(version_map_path):
    with open(version_map_path, 'r', encoding="utf8") as f:
        VERSION_MAP = json.loads(f.read())
del version_map_path



# This file helps to compute a version number in source trees obtained from
# git-archive tarball (such as those provided by githubs download-from-tag
# feature). Distribution tarballs (built by setup.py sdist) and build
# directories (produced by setup.py build) will contain a much shorter file
# that just contains the computed version number.

# This file is released into the public domain. Generated by
# versioneer-0.18 (https://github.com/warner/python-versioneer)
import os
import re

from rqsdk.const import RQDATAC_DEFAULT_ADDRESS


def init_rqdatac_env(uri):
    if uri is None:
        return

    if '@' not in uri:
        uri = "tcp://{}@{}".format(uri, RQDATAC_DEFAULT_ADDRESS)

    if not re.match(r"\w*://.+:.+@.+:\d+", uri):
        raise ValueError('invalid rqdatac uri. use user:password or tcp://user:password@ip:port')

    os.environ['RQDATAC2_CONF'] = uri



# -*- coding: utf-8 -*-
#
# Copyright 2016 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import importlib
import os
import sys

import click

from rqsdk.const import TAG_MAP, VERSION_MAP
from rqsdk.script_update import pip_install, update_bash_file


@click.group()
@click.help_option("-h", "--help")
def cli():
    pass


@cli.command()
@click.argument('rqdatac_uri', default="", nargs=1)
def license(rqdatac_uri):
    """配置rqdatac的license到环境变量"""

    def _print_rqdata_info(uri):
        import rqdatac
        from rqdatac.share.errors import AuthenticationFailed
        try:
            rqdatac.init(uri=uri)
            _info = rqdatac.user.get_quota()
            click.echo("license={uri}".format(uri=os.environ.get('RQDATAC2_CONF')))
            if _info['bytes_limit'] == 0:
                click.echo("该license无流量限制".format(_info['bytes_limit'] / 2 ** 10))
            else:
                click.echo("流量限制={}KB".format(_info['bytes_limit'] / 2 ** 10))
                click.echo("剩余流量={}KB".format((_info['bytes_limit'] - _info['bytes_used']) / 2 ** 10))
            click.echo("该license剩余天数为={}".format(_info['remaining_days']))
        except AuthenticationFailed as r:
            click.echo("登录失败 uri无法init rqdatac\n{}".format(uri))
            click.echo("请联系商务或技术支持")
            return False
        return True

    if rqdatac_uri == "info":
        uri = os.environ.get('RQDATAC2_CONF')
        if uri is None:
            click.echo("当前环境没有配置license")
        else:
            _print_rqdata_info(uri)
        return

    while True:
        if not rqdatac_uri:
            uri = os.environ.get('RQDATAC2_CONF')
            if uri is None:
                click.echo("当前环境没有配置license")
                rqdatac_uri = None
            else:
                _print_rqdata_info(uri)
        if rqdatac_uri:
            from rqsdk.utils import init_rqdatac_env
            if ":" not in rqdatac_uri:
                rqdatac_uri = "license:{}".format(rqdatac_uri)
            init_rqdatac_env(rqdatac_uri)
            rqdatac_uri = os.environ['RQDATAC2_CONF']

            if not _print_rqdata_info(rqdatac_uri):
                rqdatac_uri = ""
                continue
            if sys.platform.startswith("win"):
                os.popen("setx RQDATAC2_CONF {uri} ".format(uri=rqdatac_uri))
            else:
                update_bash_file(rqdatac_uri)
            click.echo("当前license已设为{uri}".format(uri=rqdatac_uri))
            click.echo("请重启当前的terminal".format(uri=rqdatac_uri))
            return
        rqdatac_uri = input('\n请按照【用户名:密码】的格式输入您的用户名密码或直接键入license key，输入 enter 退出 \n')
        if not rqdatac_uri:
            return


@cli.command()
@click.option('-i', '--index-url', default="https://pypi.douban.com/simple/")
def update(index_url):
    """更新rqsdk或某项产品"""
    key = ["rqdatac"]
    try:
        import rqalpha_plus
        key.append("rqalpha_plus")
    except ImportError:
        try:
            import rqfactor
            key.append("rqfactor")
        except ImportError:
            pass
        try:
            import rqoptimizer2
            key.append('rqoptimizer')
        except ImportError:
            pass

    full_name = "rqsdk[{}]".format(",".join(key))
    return pip_install(full_name, index_url)


@cli.command()
@click.argument('package_name', default="rqdatac", nargs=1)
@click.option('-i', '--index-url', default="https://pypi.douban.com/simple/")
def install(package_name, index_url):
    """安装产品"""

    if package_name == 'rqsdk':
        full_name = package_name
    elif package_name in VERSION_MAP.keys():
        from rqsdk import __version__
        full_name = "rqsdk[{}]=={}".format(package_name, __version__)
    else:
        click.echo("请输入产品名称:{}".format(list(VERSION_MAP.keys())))
        return
    return pip_install(full_name, index_url)


@cli.command()
def shell():
    """打开ipython并执行 rqdatac init"""
    import rqdatac
    rqdatac.init()
    try:
        from IPython import embed
    except ImportError:
        click.echo("请安装ipython:pip install ipython")
    else:
        embed()


@cli.command()
def version():
    """获取版本信息"""
    from rqsdk import __version__
    click.echo("rqsdk=={}".format(__version__))
    for item in VERSION_MAP['rqalpha_plus']:
        try:
            package_name = item.split("==")[0]
            _package = importlib.import_module(package_name)
            click.echo("{}=={}".format(package_name, _package.__version__))
        except:
            pass


@cli.command()
@click.option(
    '-d', '--data-bundle-path', default=os.path.expanduser('~/.rqalpha-plus'), type=click.Path(file_okay=False),
    help="bundle 目录，默认为 ~/.rqalpha-plus"
)
@click.option(
    "--minbar", multiple=True, type=click.STRING,
    help="更新分钟线数据，可选的参数值有 [{}] 或 underlying_symbol 或 order_book_id".format(", ".join(TAG_MAP))
)
@click.option(
    "--tick", multiple=True, type=click.STRING,
    help="更新tick数据，可选的参数值有 [{}] 或 underlying_symbol 或 order_book_id".format(", ".join(TAG_MAP))
)
@click.option("--daybar/--no-daybar", default=True, help="是否更新日线，默认更新日线")
@click.option("--with-derivatives", is_flag=True, default=False, help="更新分钟线和 tick 时同时更新选择的合约的衍生品数据")
@click.option('-c', '--concurrency', type=click.INT, default=3, help="并行的线程数量，需要低于 rqdatac 的最大可用连接数")
def update_data(data_bundle_path, minbar, tick, daybar, with_derivatives, concurrency):
    """更新运行回测所需的历史数据"""
    try:
        import rqdatac
        rqdatac.init()
    except ValueError as e:
        click.echo('rqdatac init failed with error: {}'.format(e))
        click.echo('请先使用rqsdk license 初始化')
        return 1

    try:
        from rqalpha_plus.bundle import update_bundle_from_rqdatac
        return update_bundle_from_rqdatac(concurrency, data_bundle_path, daybar, minbar, tick, with_derivatives)
    except ImportError:
        click.echo("""请先使用'rqsdk install rqalpha_plus'安装rqalpha_plus""")


@cli.command()
@click.option(
    '-d', '--data-bundle-path', default=os.path.expanduser('~/.rqalpha-plus'), type=click.Path(file_okay=False),
    help="bundle 目录，默认为 ~/.rqalpha-plus"
)
@click.option("--sample", is_flag=True, help="下载数据样例")
def download_data(data_bundle_path, sample=True):
    """下载bundle数据"""
    try:
        from rqalpha_plus.bundle import download_simple_bundle
        return download_simple_bundle(data_bundle_path, sample=sample)
    except ImportError:
        click.echo("""请先使用'rqsdk install rqalpha_plus'安装rqalpha_plus""")



# -*- coding: utf-8 -*-
import json
import os

from setuptools import find_packages, setup

import versioneer

version_map = {}
version_map["rqdatac"] = {
    "cryptography>=2.9.2",
    "click>=7.0",
    "pyjwt>=1.7.1",
    "patsy==0.5.1",
    "statsmodels==0.11.1",
    "rqdatac==2.8.*",
    "rqdatac_bond==0.3.*",
}
version_map["rqfactor"] = version_map["rqdatac"] | {
    "ta-lib==0.4.17",
    "rqfactor==1.0.0",
}
version_map["rqoptimizer"] = version_map["rqdatac"] | {
    "ecos==2.0.7.post1",
    "scs==2.1.1.post2",
    "cvxpy==1.0.25",
    "osqp==0.6.1",
    "rqoptimizer2==1.1.2",
}
version_map["rqalpha_plus"] = version_map["rqfactor"] | version_map["rqoptimizer"] | {
    "rqalpha==4.1.0",
    "rqalpha-mod-option==1.1.1",
    "rqalpha-mod-optimizer2==1.0.0",
    "rqalpha-mod-convertible==1.1.0",
    "rqalpha-mod-ricequant-data==2.2.2",
    "rqalpha-mod-rqfactor==1.0.0",
    "rqalpha-mod-bond==1.0.0",
    "rqalpha-mod-spot==1.0.0",
    "rqalpha-plus==4.1.*",
}
for k, v in version_map.items():
    version_map[k] = list(v)

with open(os.path.join("rqsdk", "version_map.json"), "w", encoding="utf") as f:
    f.write(json.dumps(version_map, indent=4, sort_keys=True))

extras_require = {}
extras_require["rqdatac"] = version_map["rqdatac"]
extras_require["rqfactor"] = version_map["rqfactor"]
extras_require["rqoptimizer"] = version_map["rqoptimizer"]
extras_require["rqalpha_plus"] = version_map["rqalpha_plus"]

with open('README.md', encoding="utf8") as f:
    readme = f.read()

with open('HISTORY.md', encoding="utf8") as f:
    history = f.read()

setup(
    name="rqsdk",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="Ricequant Native SDK",
    long_description="",
    author="Ricequant",
    author_email="public@ricequant.com",
    keywords="rqsdk",
    url="https://www.ricequant.com/",
    include_package_data=True,
    packages=find_packages(include=["rqsdk", "rqsdk.*"]),
    install_requires=extras_require["rqdatac"],
    python_requires=">=3.5",
    extras_require=extras_require,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Environment :: Console",
    ],
    zip_safe=False,
    entry_points={
        "console_scripts": [
            "rqsdk = rqsdk:entry_point"
        ]
    },
)



# -*- coding: utf-8 -*-
#
# Copyright 2016 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import subprocess
import sys

import click

from rqsdk.const import BASH_FILE, EXTRA_INDEX_URL


def pip_install(full_name, index_url):
    command = "install -U -i {} {} {}".format(index_url, EXTRA_INDEX_URL, full_name)
    if os.environ.get("RQSDK_PYPI"):
        command = "install -U -i {} {}".format(os.environ["RQSDK_PYPI"], full_name)
        click.echo("pip {}".format(command))
    if sys.platform.startswith("win"):
        script_update = os.path.join(os.path.dirname(os.path.abspath(__file__)), "script_update.py")
        python_path = "\"{}\"".format(sys.executable)
        script_update = "\"{}\"".format(script_update)
        subprocess.Popen("{} {} {}".format(python_path, script_update, command), shell=True)
        sys.exit(1)
    else:
        update_processing(command.split())


def update_processing(args):
    """启动子进程更新rqsdk"""
    try:
        from pip import main
    except ImportError:
        from pip._internal import main
    main(list(args))
    if sys.platform.startswith("win"):
        click.echo("请输入Enter继续...")
    return


def update_bash_file(rqdatac_uri):
    for _file in BASH_FILE:
        _path = os.path.join(os.path.expanduser("~"), _file)
        if not os.path.exists(_path):
            continue
        lines = open(_path, "r", encoding="utf8").readlines()
        flag = False
        for line in lines:
            if "export RQDATAC2_CONF=" in line:
                lines[lines.index(line)] = "export RQDATAC2_CONF={} \n".format(rqdatac_uri)
                flag = True

        if flag is False:
            lines.append("export RQDATAC2_CONF={} \n".format(rqdatac_uri))

        with open(_path, "w", encoding="utf8")as f:
            f.writelines(lines)


if __name__ == '__main__':
    update_processing(sys.argv[1:])
